<?php
class tx_mageconnect_helper
{
	const REDIRECT_METHOD_TYPO3 	= 1;
	const REDIRECT_METHOD_HEADER 	= 2;
	
	var $errors;
	
	/**
	 * _getParam
	 * Gets an parameter and decrypts it for
	 * usage
	 * 
	 * @param string Parameter
	 * @return string
	 * 
	 */	
	function _getParam($parameter = 'data')
	{
		if (t3lib_div::_GP($parameter)) {
			return t3lib_div::_GP($parameter);
		}
		return;	
	}
	
	/**
	 * getModuleConfig
	 * Gets Module Config for MAGEconnect
	 * 
	 * @param string $key
	 * @return string|bool
	 */
	function getModuleConfig($key = '')
	{
		$modConfigArray = unserialize( $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['mageconnect']);
		if ($modConfigArray[$key]) {
			return $modConfigArray[$key];
		}
		return false;
	}

	/**
	 * getTYPO3connectUrl
	 * Gets the Url for the TYPO3 connect Module that
	 * is located in the magento store
	 * 
	 * @return string
	 */
	function getTYPO3connectUrl()
	{
		return $this->getModuleConfig('magento_url') . 'customer/account/loginConnect/';
	}	
	
	/**
	 * getForgotPasswordUrl
	 * Gets the Url for the ajax forgot password controller
	 * 
	 * @return string
	 */
	function getForgotPasswordUrl()
	{
		return $this->getModuleConfig('magento_url') . 'customer/account/forgotPasswordAjax/';
	}	
	
	/**
	 * getDefaultImportAddressArrayKey
	 * Gets the array key for getting the desired
	 * address to import
	 * 
	 * @return string (either billing or shipping)
	 */
	function getDefaultImportAddressArrayKey()
	{
		$configStr = $this->getModuleConfig('import_address');
		
		if ($configStr != 'Import Default Billing Address') {
			return 'shipping';
		}
		return 'billing';
	}
		
	/**
	 * getMagentoUrl
	 * Gets the magento url from the module config
	 * 
	 * @return string
	 */
	function getMagentoUrl()
	{
		return $this->getModuleConfig('magento_url');
	}
	
	/**
	 * getMagentoBaseUrl
	 * Cleans the magento url of index.php/
	 * and returns the url
	 * 
	 * @return string
	 */
	function getMagentoBaseUrl()
	{
		$url = $this->getMagentoUrl();
		$path = pathinfo($url);
		
		return $path['dirname'];
	}
		
	/**
	 * getScriptUrl
	 * Gets the script url for this login form
	 * 
	 * @return string
	 */
	function getScriptUrl()
	{
		return t3lib_div::getIndpEnv(TYPO3_SITE_URL) . 'index.php?id=' . $GLOBALS['TSFE']->id;
	}

	/**
	 * isInDebugMode
	 * Determines if extension is currently 
	 * in debug mode
	 * 
	 * @return bool
	 */
	function isInDebugMode()
	{
		return $this->getModuleConfig('debug_mode');
	}
	
	/**
	 * getRedirectMethod
	 * Gets the configuration of the redirect method
	 * 
	 * @return string the method
	 */
	function getRedirectMethod()
	{
		$config = $this->getModuleConfig('redirect_method');
		
		switch ($config) {
			case 't3lib_utility_Http::redirect':
				return self::REDIRECT_METHOD_TYPO3;	
			case 'header(location:)':
			default:
				return self::REDIRECT_METHOD_HEADER;

		}
	}

	/**
	 * getAttributeValue
	 * Gets an attribute value from an
	 * serialized string
	 * 
	 * @param string $attributeCode
	 * @param string $serializedStr
	 * @return string
	 */	
	function getAttributeValue($attributeCode, $serializedStr)
	{
		$_attributes = unserialize($serializedStr);
		
		foreach ($_attributes as $_code=>$_value) {
			if ($_code === $attributeCode) {
				return $_value;
			}
		}
		return '';
	}
	
	/**
	 * getMarkers
	 * Generate an marker array by an given data
	 * array
	 * 
	 * @param 	array 	$data
	 * @return	array
	 */
	function getMarkers($data = array(), $prefix = null) 
	{
		$markers = array();
			
		if ($prefix) {
			$prefix = $prefix . '_';
		}
			
		foreach ($data as $_field=>$_value) {
			if (!is_array($_value)) {
				$_marker = '###' . $prefix . strtoupper($_field) . '###';	
				$markers[$_marker] = $_value;
			}
		}
		
		return $markers;
	}
	
	/**
	 * displayError
	 * Displays an errormessage
	 *
	 * @return	string		the error message
	 */
	function displayError($errormsg) 
	{
		$flashMessage = t3lib_div::makeInstance(
			't3lib_FlashMessage',
			$errormsg,
			'tx_mageconnect Error',
			t3lib_FlashMessage::ERROR
		);
		return $flashMessage->render();
	}
		
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_helper.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_helper.php']);
}
