<?php
/**
 * MAGECONNECT
 * Connecting TYPO3 and Magento
 * 
 * Fetch Product Class
 * 
 * This class fetches product details from
 * TYPO3 and Magento
 * 
 * @author Bastian 'Aggy' Zagar
 * @version 0.1
 *
 */
require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_soap.php');
require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_category.php');
require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_registry.php');

class tx_mageconnect_product
{
	/**
	 * Soap Class
	 * @var tx_mageconnect_soap
	 */	
	private $_soap;
	
	private $cObj;
	public  $conf;
	private $controller;
	
	function __construct()
	{
		$this->cObj 		= tx_mageconnect_registry::get('cObj');
		$this->conf 		= tx_mageconnect_registry::get('conf');
		$this->controller 	= tx_mageconnect_registry::get('controller');
		$this->controller->pi_loadLL();
	}
	
	function __destruct()
	{
		
	}
	
	/**
	 * _connect
	 * Connect to webservice
	 */
	function _connect()
	{
		$soapUrl 	= $this->controller->_helper->getModuleConfig('magento_url');
		$soapUser 	= $this->controller->_helper->getModuleConfig('magento_api_username');
		$soapKey 	= $this->controller->_helper->getModuleConfig('magento_api_key');
		
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	/**
	 * getImage
	 * Gets an resized image; 
	 * Looks, what display type has been chosen
	 * 
	 * @param string $image
	 * @return string
	 */
	function getImage($image)
	{
		// Determine image type
		$displayType = $this->_getFlexformConfig('display_type', 'product_setting');
		
		// Check flexform config and fetch settings for image type
		switch ($displayType) {
			
				// Products list
				case 'display_list':
					$imageType = 'listImage';
					break;
				// Products grid
				case 'display_grid':
					$imageType = 'gridImage';
					break;	
				// Single product
				case 'display_single':
					$imageType = 'singleImage';
					break;	
				// Product details
				case 'display_detail':
					$imageType = 'detailImage';
					break;	
				case 'dynamic_details':
					$imageType = 'detailImage';
					break;	
				default:
					$imageType = 'listImage';
					break;
				default:
					$imageType = 'listImage';
					break;
		}		
		
		$img = array();
		$img = $this->conf[$imageType.'.'];
		$img['file'] = $this->conf['imagePath'].DIRECTORY_SEPARATOR.$image;
		
		$image = $this->cObj->IMAGE($img);
		return $image;
	}
	
	/**
	 * getProductMarkers
	 * Gets all possible product
	 * markers
	 * 
	 * @param array $product Product Data Array
	 * @return array
	 */
	public function getProductMarkers($product)
	{
		$attributes = array();
		$attributes = unserialize($product['attributes']);
			
		foreach ($attributes as $_code=>$_val) {
			if (is_array($_val)) {
				unset($attributes[$_code]);
			}
		}
			
		$attributeMarkers = $this->controller->_helper->getMarkers($attributes, 'DETAIL');
			
		unset($product['attributes']);
		$markers = $this->controller->_helper->getMarkers($product);
		
		// Product URL Marker
		if ($this->isDynamicDetailView()) {
			
			$params = array('&tx_mageconnect[entity_id]' => $attributes['entity_id']);
			$markers['###DETAIL_VIEW_URL###'] = $this->cObj->getTypoLink_URL($this->getDynamicDetailViewPid(), $params);
			
		} else {
			if (array_key_exists('url_path', $attributes)) {
				$markers['###DETAIL_VIEW_URL###'] = $this->controller->_helper->getMagentoUrl() . $attributes['url_path'];
			}
		}
		
		// Product Image Marker
		if (array_key_exists('image', $product)) {
			$markers['###IMAGE###'] = $this->getImage($product['image']);
		}
		
		if (isset($attributeMarkers)) {
			$markers = t3lib_div::array_merge_recursive_overrule($attributeMarkers, $markers);
		}
		
		// SPECIAL PRODUCTS MARKERS
		$markers['###STOCK###']	= $this->controller->pi_getLL('template_label_in_stock') . ' ' . (int)$product['stock_qty'];
		
		if ($product['stock_qty'] > 0) {
			$markers['###AVAILABILITY###'] = '<span class=\'available\'>'.$this->controller->pi_getLL('template_label_available').'</span>';
		} else {
			$markers['###AVAILABILITY###'] = '<span class=\'not_available\'>'.$this->controller->pi_getLL('template_label_not_available').'</span>';
		}
		
		if ($product['special_price'] != '' || !empty($product['special_price'])) {
			$markers['###PRICE###'] = '<span class="old_price">'.$product['price'].'</span>';
			$markers['###SPECIAL_PRICE###'] = $product['special_price'];
			$markers['###CLASS###'] = 'special';
		} else {
			$markers['###PRICE###'] = $product['price'];
			$markers['###SPECIAL_PRICE###'] = $product['special_price'];
			$markers['###CLASS###'] = 'normal';
		}
		
		$markers = t3lib_div::array_merge_recursive_overrule($markers, $this->getGlobalMarkers());
		return $markers;
	}
	
	/**
	 * getGlobalMarkers
	 * Gets global markers for every need
	 * 
	 * @return array
	 */
	function getGlobalMarkers()
	{
		$globMarkers = array();
		$globMarkers['###MAGENTO_BASE_URL###'] 				= $this->controller->_helper->getMagentoUrl();
		$globMarkers['###MAGENTO_MEDIA_URL###'] 			= trim($this->controller->_helper->getMagentoUrl(),'/') .'/'.'media';
		$globMarkers['###MAGENTO_PRODUCT_IMAGE_URL###'] 	= trim($this->controller->_helper->getMagentoUrl(),'/') .'/'.'media/catalog/product';
		$globMarkers['###MAGENTO_ADDTOCART_URL###'] 		= trim($this->controller->_helper->getMagentoUrl(),'/') .'/'.'checkout/cart/add/product/';
		
		// Labels
		$globMarkers['###LABEL_ADDTOCART###'] 				= $this->controller->pi_getLL('template_label_add_to_cart');
		$globMarkers['###LABEL_AVAILABILITY###'] 			= $this->controller->pi_getLL('template_label_availability');
		$globMarkers['###LABEL_AVAILABLE###'] 				= $this->controller->pi_getLL('template_label_available');
		$globMarkers['###LABEL_NOT_AVAILABLE###'] 			= $this->controller->pi_getLL('template_label_not_available');
		$globMarkers['###LABEL_IN_STOCK###'] 				= $this->controller->pi_getLL('template_label_in_stock');
		$globMarkers['###LABEL_PRICE###'] 					= $this->controller->pi_getLL('template_label_price');
		$globMarkers['###LABEL_SPECIAL_PRICE###'] 			= $this->controller->pi_getLL('template_label_special_price');
		$globMarkers['###LABEL_ADDITIONAL_INFORMATION###']  = $this->controller->pi_getLL('template_label_additional_information');
		
		return $globMarkers;
	}	
	
	/**
	 * getProductsFromDb
	 * Gets product details from database
	 * 
	 * @param array $productIds
	 * @param string $store Store View Code
	 * @return array
	 */
	function getProductsFromDb($productIds = array(), $store = '')
	{
		$products = array();
		$pid = $this->_getFlexformConfig('productsPid', 'general_setting');
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 							// SELECT
															 'tx_mageconnect_products', 	// FROM
															 'product_id IN ('.implode(',',$productIds).') 
															 	AND store_view = \''.$store.'\'
															 	AND hidden = \'0\' 
															 	AND deleted = \'0\'
															 	AND pid = \''.$pid.'\'',	// WHERE
															 'product_id',					// GROUP BY
															 '',							// ORDER BY
															 ''								// LIMIT
															);
		if ($dbResource) {
			
			while($_product = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource)) {
				$products[] = $_product;
			}
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
			
		}
		return $products;
	}
	
	/**
	 * getAllProductsFromDb
	 * Gets all existing products from database
	 * 
	 * @param string $store Store View Code
	 * @return array
	 */
	function getAllProductsFromDb($store = '')
	{
		$products = array();
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('product_id,refresh', 'tx_mageconnect_products', 'store_view = \''.$store.'\'');
		if ($dbResource) {
			
			$rows = array();
			while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource)) {
				$rows[] = $row;
			}
		}
		foreach ($rows as $_row) {
			
			$product = array(
				'product_id' => $_row['product_id'],
				'refresh'	 => $_row['refresh']
			);
			
			$products[] = $product;
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
		}
		return $products;
	}
	
	/**
	 * getExistingProducts
	 * Gets all existing products in database
	 * 
	 * @param string $store Store View Code
	 * @return array
	 */
	function getExistingProducts($store = '')
	{
		$existing = array();
		
		$productsDb = $this->getAllProductsFromDb($store);
		
		foreach ($productsDb as $_p) {
			if ($_p['refresh'] == '0') {
				$existing[] = $_p['product_id'];
			}
			
		}
		return $existing;
	}	
	
	
	
	/**
	 * getRefreshableProducts
	 * Gets all existing products that need to be refreshed from the database
	 * 
	 * @return array
	 */
	function getRefreshableProducts($store = '')
	{
		$refreshable = array();
		
		$productsDb = $this->getAllProductsFromDb($store);
		
		foreach ($productsDb as $_p) {
			if ($_p['refresh'] == '1') {
				$refreshable[] = $_p['product_id'];
			}
			
		}
		return $refreshable;
	}
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		return $this->controller->pi_getFFValue($this->cObj->data['pi_flexform'], $field, $type); 
	}	
	
	/**
	 * isDynamicDetailView
	 * Checks if dynamic detail view is activated
	 * 
	 * @return bool
	 */
	function isDynamicDetailView()
	{
		if ($this->getDynamicDetailViewPid()) {
			return true;
		}
		return false;
	}
	
	/**
	 * isAllowedDynamicDetailView
	 * Checks if entity id is allowed for dynamic detail view
	 * 
	 * @param int $entityId Id of the product
	 * @return bool
	 */
	function isAllowedDynamicDetailView($entityId)
	{
		$allowedIds = $this->getFlexformAllowedProducts();
		
		if (in_array($entityId, $allowedIds) || !sizeof($allowedIds)) {
			return true;
		}
		return false;
	}
	
	/**
	 * getFlexformAllowedProducts
	 * Determine the products ids that have been
	 * allowed for detail view in the backend
	 * 
	 * @return array
	 */
	function getFlexformAllowedProducts()
	{
		$products = $this->_getFlexformConfig('allowed_products', 'product_setting');
		$products = t3lib_div::trimExplode(',',$products,1);
		
		return $products;
	}	
	
	/**
	 * getSelectedProducts
	 * Determine the products ids that have been
	 * set in the backend
	 * 
	 * @return array
	 */
	function getFlexformSelectedProducts()
	{
		$products = $this->_getFlexformConfig('product_single', 'product_setting');
		$products = t3lib_div::trimExplode(',',$products,1);
		
		return $products;
	}	
	
	/**
	 * getFlexformSelectedCategoryId
	 * Gets the category id that was selected in the flexform
	 * 
	 * @return int
	 */
	function getFlexformSelectedCategoryId()
	{
		return $this->_getFlexformConfig('category_single', 'product_setting');
	}
	
	/**
	 * getFlexformStoragePid
	 * Gets the storage pid of the category sysfolder
	 * 
	 * @return int
	 */
	function getFlexformStoragePid()
	{
		return $this->_getFlexformConfig('categoriesPid', 'general_setting');
	}
	
	/**
	 * getFlexformCategoryProducts
	 * Determine the product ids that come from
	 * a category, that was set in the backend
	 * 
	 * @return array
	 */
	function getFlexformCategoryProducts()
	{
		// Category Class Instance
		$categoryClass = t3lib_div::makeInstance('tx_mageconnect_category');
		
		$categoryId = $this->getFlexformSelectedCategoryId();
		$storeview = $this->getFlexformStoreView();
		$storagePid = $this->getFlexformStoragePid();
		
		$ids = array();
		
		$category = $categoryClass->getCategoryById($categoryId, $storeview, $storagePid);
		
		if ($category && array_key_exists('uid',$category)) {
			$ids = explode(',', $category['product_ids']);
		}		
		return $ids;
	}
	
	/**
	 * getFlexformStoreView
	 * Gets the flexform storeview setting
	 * 
	 * @return string
	 */
	function getFlexformStoreView()
	{
		$storeview_code = $this->_getFlexformConfig('store_view_code', 'general_setting');
		return (string)$storeview_code;
	}
	
	/**
	 * getDynamicDetailViewPid
	 * Gets the dynamic detail view pid from
	 * the flexform configuration
	 * 
	 * @return int
	 */
	function getDynamicDetailViewPid()
	{
		$pid = $this->_getFlexformConfig('dynamic_detail_pid', 'product_setting');
		
		if ($pid > 0) {
			return $pid;
		}
		return;
	}
	
		
	/**
	 * importImage
	 * Imports an image from a given image path
	 * 
	 * @param string $img
	 */	
	function importImage($img) 
	{
		$mediaUrl = trim($this->controller->_helper->getMagentoBaseUrl(),'/') .'/'.'media/catalog/product';
		
		$uploadPath = $this->conf['imagePath'] . DIRECTORY_SEPARATOR;
		
		$file = basename($img);
		$imgfile = $uploadPath . $file;
		
		if(!is_file($imgfile) || !filesize($imgfile)) {
			
			$img 	= file_get_contents($mediaUrl . $img);
			
			if (!t3lib_div::writeFile($imgfile, $img)) {
				return false;
			}
		}
		return $file;
	}	
		
	/**
	 * importProducts
	 * Import given product ids to database for
	 * better performance
	 * 
	 * @param $productIds array
	 * @param $store string Store View Code
	 * @return bool
	 */	
	function importProducts($productIds = array(), $store = '') 
	{
		$this->_connect();
		
		$existingIds = $this->getExistingProducts($store);
		$refreshableIds = $this->getRefreshableProducts($store);
		
		$status = null;
		
		foreach ($productIds as $_id) {
			
			$productData = $this->_soap->getProductById($_id, $store);

			// Product Image
			$image = '';
			if (array_key_exists('image', $productData)) {
				if ($productData['image'] != 'no_selection') {
					$image = $this->importImage($productData['image']);
				}
			}

			if ( t3lib_div::inArray($existingIds, $_id) || 
				 t3lib_div::inArray($refreshableIds, $_id)
			) {
				$product = array(
					'pid'					=> $this->_getFlexformConfig('productsPid', 'general_setting'),
					'tstamp'				=> time(),
					'product_id'			=> $productData['product_id'],
					'store_view'			=> $store,
					'sku'					=> $productData['sku'],
					'name'					=> $productData['name'],
					'short_description'		=> $productData['short_description'],
					'description'			=> $productData['description'],
					'price'					=> $productData['price'],
					'special_price'			=> $productData['special_price'],
					'stock_qty'				=> $productData['qty'],
					'image'					=> $image,
					'attributes'			=> serialize($productData),
				);
				$result = $this->updateProduct($_id, $product, $store);
 				
			} else {
				$product = array(
					'pid'					=> $this->_getFlexformConfig('productsPid', 'general_setting'),
					'crdate'				=> time(),
					'product_id'			=> $productData['product_id'],
					'store_view'			=> $store,
					'sku'					=> $productData['sku'],
					'name'					=> $productData['name'],
					'short_description'		=> $productData['short_description'],
					'description'			=> $productData['description'],
					'price'					=> $productData['price'],
					'special_price'			=> $productData['special_price'],
					'stock_qty'				=> $productData['qty'],
					'image'					=> $image,
					'attributes'			=> serialize($productData)
				);
				$result = $this->insertProduct($product);
			}
			if ($result !== true) {
				$status = false;
			}
		}
		return ($status === null)?true:false;
	}	

	/**
	 * insertProduct
	 * Inserts an product to database
	 * 
	 * @param array $productData
	 * @return bool
	 */
	public function insertProduct($productData)
	{
		try {
			$insertQuery = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_mageconnect_products', $productData);
		
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}
	
	/**
	 * updateProduct
	 * Updates an product in the database
	 * 
	 * @param int $productId
	 * @param array $productData
	 * @param string $store Store View Code
	 * @return bool
	 */
	public function updateProduct($productId, $productData, $store = '')
	{
		try {
			$updateQuery = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_mageconnect_products', 'product_id=\''. $productId . '\' AND store_view = \''. $store . '\'', $productData);
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}		
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_product.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_product.php']);
}
