<?php
/**
 * MAGECONNECT
 * Connecting TYPO3 and Magento
 * 
 * Fetch User Class
 * 
 * This class fetches user details from
 * TYPO3 and Magento
 * 
 * @author Bastian 'Aggy' Zagar
 * @version 0.1
 *
 */

require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_soap.php');
require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_registry.php');

class tx_mageconnect_user
{
	/**
	 * Soap Class
	 */	
	private $_soap;

	private $cObj;
	public  $conf;
	private $controller;
	
	function __construct()
	{
		$this->cObj 		= tx_mageconnect_registry::get('cObj');
		$this->conf 		= tx_mageconnect_registry::get('conf');
		$this->controller 	= tx_mageconnect_registry::get('controller');
		$this->controller->pi_loadLL();
		
		$soapUrl 	= $this->controller->_helper->getModuleConfig('magento_url');
		$soapUser 	= $this->controller->_helper->getModuleConfig('magento_api_username');
		$soapKey 	= $this->controller->_helper->getModuleConfig('magento_api_key');
			
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	function __destruct()
	{
		
	}
	
	/**
	 * getMagentoUserByEmail
	 * Get a magento user from soap by a given 
	 * email address
	 * 
	 * @param string $email E-Mail Address
	 * @return array|bool
	 */
	public function getMagentoUserByEmail($email)
	{
		if (!$email) {
			return false;
		}

		$customer = $this->_soap->getCustomerByEmail($email);
		
		if ($customer) {
			return $customer;
		}
		return false;
	}
	
	/**
	 * insertTYPO3User
	 * Inserts an TYPO3 User by Userdata Array
	 * @param array $userdata
	 * @return bool
	 */
	public function insertTYPO3User($userdata)
	{
		try {
			$insertQuery = $GLOBALS['TYPO3_DB']->exec_INSERTquery('fe_users', $userdata);
		
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}
	
	/**
	 * updateTYPO3UserByUsername
	 * Updates an TYPO3 User by username
	 * 
	 * @param string $username
	 * @param array $updatedata
	 * @return bool
	 */
	public function updateTYPO3User($username, $updatedata)
	{
		try {
			$updateQuery = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('fe_users', 'username=\''. $username . '\'', $updatedata);
		
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}	
	
	/**
	 * getTYPO3UserById
	 * Gets an TYPO3 User by uid
	 * 
	 * @param int $uid
	 * @return array
	 */
	public function getTYPO3UserById($uid)
	{
		$userData = false;
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'fe_users', 'uid='.intval($uid).' ');
		if ($dbResource) {
			$userData = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource);
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
		}
		return $userData;
	}
	
	/**
	 * getTYPO3UserByUsername
	 * Gets an TYPO3 User by username
	 * 
	 * @param string $username
	 * @return array
	 */
	public function getTYPO3UserByUsername($username)
	{
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', 'fe_users', 'username LIKE \''.$username.'\' ');
		
		if ($dbResource) {
			
			$userData = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource);
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
			
			if ($userData) {
				return $userData;
			}
		}
		return false;
	}	

	/**
	 * getPasswordHash
	 * Gets the stored password hash from a username
	 * 
	 * @param string $username
	 * @return array
	 */
	public function getPasswordHash($username)
	{
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('password', 'fe_users', 'username = \''.$username.'\' ');
		
		if ($dbResource) {
			
			$data = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource);
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
			
			if ($data) {
				return $data['password'];
			}
		}
		return false;
	}	
	
	/**
	 * magentoPasswordCompare
	 * Compares a given plain password with a given
	 * hashed magento password
	 * 
	 * @param string $plainPassword
	 * @param string $magentoHash
	 * @return bool
	 */
	public function magentoPasswordCompare($plainPassword, $magentoHash)
	{
		$salt = explode(":", $magentoHash);
		$salt = $salt[1];
	
		$hash = md5($salt . $plainPassword) . ":" . $salt;
	
		if( $hash == $magentoHash ) {
			return true;	
		}
	
		return false;
	}
	
	/**
	 * checkTYPO3UserExists
	 * Checks if a TYPO3 User exists with
	 * a given username
	 * 
	 * @param string $username
	 * @return bool
	 */
	public function checkTYPO3UserExists($username)
	{
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', 'fe_users', 'username LIKE \''.$username.'\' ');
		
		if ($dbResource) {
			
			$userData = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource);
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
			
			if ($userData) {
				return true;
			}
		}
		return false;
	}	
	
	
	/**
	 * getMagentoCustomerAttributesFromTYPO3UserById
	 * Gets the Magento Customer Attributes by a given TYPO3 uid
	 * 
	 * @param $uid
	 * return array
	 */
	public function getMagentoCustomerAttributesFromTYPO3UserById($uid)
	{
		$attributeData = array();
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('tx_mageconnect_magento_customer_attr', 'fe_users', 'uid='.intval($uid).' ');
		
		if ($dbResource) {
			$magentoData = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource);
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
			
			$magentoData = explode('*', $magentoData['tx_mageconnect_magento_customer_attr']);
			
			for ($i = 0; $i < count($magentoData) - 1; $i++) {
				
				$statements = t3lib_div::trimExplode('::', $magentoData[$i]);
				
				$attributeData[$i]['attribute_code']  = $statements[0];
				$attributeData[$i]['frontend_label']  = $statements[1];
				$attributeData[$i]['attribute_value'] = $statements[2];
			}
		}
		return $attributeData;
	}	
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_user.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_user.php']);
}
